package jp.gr.java_conf.abagames.brick;

import jp.gr.java_conf.abagames.brick.shapes.*;
import jp.gr.java_conf.abagames.util.*;
import jp.gr.java_conf.abagames.util.gl4j.*;
import gl4java.*;

/**
  block crasher - ball
  */
public class Ball extends TrnsObjDSFix implements IMove {
  private BallShape ballShape = new BallShape();

  private boolean existf;
  public float mx, my;
  private float speed;
  private Racket racket;
  private BlockManager blockManager;
  private BallManager ballManager;
  private GameManager gameManager;
  public int cnt;

  public Ball(
   Racket racket, BlockManager blockManager, BallManager ballManager,
   GameManager gameManager, GLFunc gl) {
    super(gl);
    this.shape = ballShape;
    existf = false;
    this.racket = racket;
    this.blockManager = blockManager;
    this.ballManager = ballManager;
    this.gameManager = gameManager;
  }

  public void clear() {
    existf = false;
  }
  
  public final void set(float x, float y, int deg, float speed, int cnt) {
    this.x = x; this.y = y;
    this.speed = speed;
    mx = SCTable.sin[deg]*speed; my = -SCTable.cos[deg]*speed;
    existf = true;
    deadf = false;
    setCount(cnt);
  }

  private void setColor() {
    if ( cnt < 0 ) ballShape.setColor(1, 0.5f, 0.5f);
    else if ( cnt > 99999 ) ballShape.setColor(1, 1, 0.5f);
    else if ( cnt > 200 ) ballShape.setColor(1, 1, 1);
    else {
      float c = cnt*0.0025f+0.5f;
      ballShape.setColor(c, c, 1);
    }
  }

  public final void setCount(int c) {
    cnt = c;
    setColor();
  }

  private final int[][] rflDeg = {
    {12, 24, 36, 48, 60, 72},
    {180-12, 180-24, 180-36, 180-48, 180-60, 180-72},
  };

  public static final float RKTRFLWDT = 2;

  private void setReflect(float dst, int type, float px, float py) {
    if ( dst > RKTRFLWDT ) {
      my = -my;
    } else {
      int deg = rflDeg[type][(int)(dst/(RKTRFLWDT/6))];
      mx = SCTable.cos[deg]*speed; my = SCTable.sin[deg]*speed;
      x = px+mx;
    }
    y = py+my;
    if ( cnt < 0 ) cnt = -cnt;
    setColor();
    if ( blockManager.rndClr && cnt>99999 ) {
      blockManager.ballReflected();
    }
    gameManager.playSe(1);
  }

  private void disappear() {
    existf = false;
    ballManager.ballNmb--;
  }

  private boolean deadf;
  public float px, py;

  public final void move() {
    px = x; py = y;
    x += mx; y += my;
    if ( x <= -Wall.WIDTH || x >= Wall.WIDTH ) {
      if ( cnt == 0 ) existf = false;
      mx = -mx; x = px+mx;
    }
    if ( !deadf && y <= -Wall.HEIGHT+Racket.YPOS ) {
      if ( x < racket.x ) {
        setReflect(racket.x-x, 0, px, py);
      } else if ( x > racket.x+racket.width ) {
        setReflect(x-racket.x-racket.width, 1, px, py);
      } else {
        deadf = true;
      }
    }
    if ( y >= Wall.HEIGHT ) {
      if ( cnt == 0 ) existf = false;
      my = -my; y = py+my;
    }
    if ( y <= -Wall.HEIGHT ) {
      disappear();
      gameManager.ballDropped(x);
    }

    if ( cnt > 0 ) {
      if ( blockManager.checkHit(this) ) {
        gameManager.playSe(0);
      }
      cnt--;
      if ( (cnt&15)==0 ) setColor();
    } else if ( cnt == 0 ) {
      if ( blockManager.checkHit(this) ) {
        gameManager.playSe(0);
        existf = false;
      }
    }
  }

  public final boolean exists() {
    return existf;
  }
}
