package jp.gr.java_conf.abagames.brick;

import jp.gr.java_conf.abagames.brick.shapes.*;
import jp.gr.java_conf.abagames.util.*;
import jp.gr.java_conf.abagames.util.gl4j.*;
import jp.gr.java_conf.abagames.util.jmf.*;
import java.util.Random;

/**
  manage game status, frame, ui, ...
  */
public class GameManager implements ITime, IGameManager {

  private SaveDataManager sdManager;
  private SaveData saveData;

  private SceneUI sceneUI;
  private SePlayer sePlayer;

  private boolean keyPriorityf, slowf, bgefcf, joystickf, pingsef;
  private int frameSkip;
  private boolean bgmf, sef;

  public GameManager(boolean kpf, int frmSkp, boolean slowf, boolean grdf,
   boolean bgefcf, boolean bgmf, boolean sef, boolean pingsef,
   boolean joystickf) {
    keyPriorityf = kpf;
    frameSkip = frmSkp;
    this.slowf = slowf; this.bgefcf = bgefcf;
    this.bgmf = bgmf; this.sef = sef; this.pingsef = pingsef;
    this.joystickf = joystickf;
    ShapeProperties.grdf = grdf;

    sdManager = new SaveDataManager();
    sdManager.load();
    saveData = sdManager.getSaveData();

    sceneUI = new SceneUI(this, saveData.sceneUIRect);
    sceneUI.show();
  }

  public final int DURATION_DEFAULT = 33;
  private int duration = DURATION_DEFAULT;
  private Timer timer;

  private BlockManager blockManager;
  private BallManager ballManager;
  private Wall[] walls = new Wall[3];
  private Racket racket;

  private final int EFCLN_NMB = 32;
  private EffectLine[] efcLns = new EffectLine[EFCLN_NMB];

  private DemoManager demoManager;

  private int frmCnt;

  private final String BGM_FILE = "brk2.wav";

  public void init() {
    System.out.println("\n\nNow loading _bong_ ... please wait.");

    scene = sceneUI.getScene();
    if ( joystickf ) {
      sceneUI.initJoystick();
      sceneUI.enable(joystickf);
    }
    racket = new Racket(sceneUI, scene.glFunc);
    blockManager = new BlockManager(racket, this, scene.glFunc);
    ballManager = new BallManager(racket, blockManager, this, slowf, scene.glFunc);
    blockManager.bindBallManager(ballManager);
    blockManager.bindStageData();

    for ( int i=0 ; i<walls.length ; i++ ) {
      walls[i] = new Wall(scene.glFunc);
    }
    walls[0].set(0, Wall.HEIGHT+WallShape.WIDTH, 0, Wall.WIDTH+WallShape.WIDTH*2);
    walls[1].set(-Wall.WIDTH-WallShape.WIDTH-0.05f, 0, 90, Wall.HEIGHT);
    walls[2].set( Wall.WIDTH+WallShape.WIDTH+0.05f, 0, 270, Wall.HEIGHT);
    for ( int i=0 ; i<efcLns.length ; i++ ) {
      efcLns[i] = new EffectLine(scene.glFunc);
    }

    scLts = new Letters(scene.glFunc);
    tmLts = new Letters(scene.glFunc);
    htLts = new Letters(scene.glFunc);
    taLts = new Letters(scene.glFunc);
    hsLts = new Letters(scene.glFunc);
    mfLts = new Letters(scene.glFunc);
    psLts = new Letters(scene.glFunc);
    scLts.setColor(0.3f, 0.7f, 0.9f);
    tmLts.setColor(0.3f, 0.7f, 0.4f);
    htLts.setColor(0.7f, 0.4f, 0.3f);
    taLts.setColor(0.4f, 0.7f, 0.3f);
    hsLts.setColor(0.7f, 0.5f, 0.9f);
    mfLts.setColor(0.3f, 0.7f, 0.7f);
    psLts.setColor(1, 1, 1);

    demoManager = new DemoManager(
     this, blockManager, ballManager, racket, sceneUI, scene.glFunc);

    try {
      sePlayer = new SePlayer(pingsef);
    } catch ( NoClassDefFoundError e ) {
      sef = false;
    }
    loadBgm(BGM_FILE);

    timer = new Timer(this);
    timer.activate(duration, keyPriorityf);

    demoManager.demoStart();
  }

  public void destroy() {
    if ( bgmf ) {
      bgm.close();
    }
    if ( timer != null ) {
      timer.deactivate();
    }
    if ( scene != null ) {
      scene.destroy();
    }

    saveData.sceneUIRect = sceneUI.getBounds();
    sdManager.save();

    try {
      Thread.sleep(duration*2);
    } catch ( Exception e ) {}
    //notifyFrame();

    System.exit(1);
  }

  private MdPlayer bgm;

  private void loadBgm(String filename) {
    /*if ( !saveData.bgmf ) {
      bgmf = false;
      return;
    }*/
    if ( !bgmf ) return;
    try {
      bgm = new MdPlayer(filename, true);
      bgmf = true;
    } catch ( Exception e ) {
      e.printStackTrace();
      /*saveData.bgmf = */bgmf = false;
    } catch ( NoClassDefFoundError e ) {
      /*saveData.bgmf = */bgmf = false;
    }
  }

  public final void playSe(int nmb) {
    if ( /*!saveData.*/!sef ) return;
    sePlayer.play(nmb);
  }

  private int time;
  private int seCnt;
  private int score;
  private Letters scLts, tmLts, htLts, taLts, hsLts, mfLts, psLts;
  private boolean gmStt;
  private final float tmlX = 3, tmlY = 10;
  private float tmlMv;
  private final float TML_SIZE = 1;
  private boolean pausef;

  public void gameInit() {
    setDuration(DURATION_DEFAULT);
    tmLts.clear();
    htLts.clear();
    taLts.clear();
    mfLts.clear();
    hsLts.start("", -2, 11.5f, 2f, 0.5f, 999999999);
    hsLts.add(saveData.hiscore);
    aelx = elx = aely = ely = 0; aelz = elz = EFCLN_DST;
    tmlMv = 0;
    elCngCnt = 200;
    racket.init();
    blockManager.clear();
    ballManager.clear();
  }

  public void demoStart() {
    gameInit();
    ballManager.addBall(0, 0, 36, 0.3f, 999999999);
    time = 1000*60*7;
    System.gc();
  }

  public void gameStart() {
    gameInit();
    scLts.clear();
    blockManager.init();
    ballManager.addBall(0, 0, 36, 0.3f, 999999999);
    time = 1000*60*7;
    seCnt = 0;
    score = 0;
    gmStt = false;
    pausef = false;
    scLts.start("0", -9f, 10, 2f, 0, 20f, 10f, 1, 999999999);
    tmLts.start("", tmlX, tmlY, 2f, 0, 20f, 350f, TML_SIZE, 999999999);
    htLts.start("", -10f, 5, 1f, 0, 20f, 20f, 0.8f, 0);
    taLts.start("",  10f, 5, 1f, 0, 20f, 340f, 0.8f, 0);
    mfLts.start("", 0, 0, 0, 1, 0);
    psLts.start("PAUSE", 0, 0, 0, 0.5f, 0);
    tmlMv = 0;
    hitNmb = hitTime = 0;
    System.gc();
    if ( bgmf ) bgm.play();
    playSe(4);
  }

  public void ballReflected() {
    if ( !gmStt ) {
      gmStt = true;
      playSe(5);
    }
  }

  public void ballDropped(float x) {
    tmlMv = 1.6f;
    if ( !demoManager.demof ) {
      time -= 5000;
      mfLts.set(x, -Wall.HEIGHT, 0.25f);
      mfLts.add("-5 SEC");
      mfLts.time = 30;
    }
    playSe(3);
  }

  public final void setDuration(int duration) {
    this.duration = duration;
    timer.setDuration(duration);
  }

  public final void addScore(int as) {
    if ( demoManager.demof ) return;
    score += as;
    scLts.set();
    scLts.add(score);
    if ( score > saveData.hiscore ) {
      hsLts.set();
      hsLts.add(score);
    }
  }

  private int hitNmb, hitTime;

  private void decHitTime() {
    if ( hitTime > 0 ) {
      hitTime--;
      if ( hitTime <= 0 ) {
        if ( hitNmb >= 20 ) {
          int at = hitNmb/20;
          time += at*1000;
          taLts.set();
          taLts.add(at);
          taLts.add("SEC");
          taLts.time = 60;
          taLts.x = 6;
          taLts.xScl = taLts.yScl = 0;
        }
        hitNmb = 0;
      }
    }
  }

  private void addHitNmb() {
    if ( demoManager.demof ) return;
    hitNmb++;
    hitTime = 5;
    if ( hitNmb >= 20 ) {
      htLts.set();
      htLts.add(hitNmb);
      htLts.add("HIT");
      htLts.time = 60;
    }
  }

  public final void breakBlock() {
    addScore(10);
    addHitNmb();
  }

  public final void hitBlock() {
    addScore(1);
    addHitNmb();
  }

  synchronized private void waitFrame() {
    try {
      wait();
    } catch ( InterruptedException e ) {}
  }

  synchronized final public void notifyFrame() {
    notifyAll();
  }

  private Scene scene;

  private void drawWalls() {
    for ( int i=0 ; i<walls.length ; i++ ) {
      walls[i].draw();
    }
  }

  private void drawEffectLines() {
    if ( !bgefcf ) return;
    scene.glFunc.glPushMatrix();
    if ( tmlMv != 0 ) {
      float x = rnd.nextFloat()*tmlMv*3;
      float y = rnd.nextFloat()*tmlMv*3;
      float z = rnd.nextFloat()*tmlMv*3;
      scene.setViewpoint(elx+x, ely+y, elz+z, x, y, z);
    } else {
      scene.setViewpoint(elx, ely, elz, 0, 0, 0);
    }
    for ( int i=efcLns.length-1 ; i>=0 ; i-- ) {
      efcLns[i].draw();
    }
    scene.glFunc.glPopMatrix();
  }

  private float elx, ely, elz;

  public final void display() {
    drawEffectLines();

    scene.glFunc.glPushMatrix();
    scene.setViewpoint(0, -6.0f, 14.0f,  0, -2.5f, 0);

    drawWalls();
    blockManager.draw();
    racket.draw();
    ballManager.draw();
    hsLts.draw();
    scLts.draw();
    tmLts.draw();
    if ( htLts.exists() ) htLts.draw();
    if ( taLts.exists() ) taLts.draw();
    if ( mfLts.exists() ) mfLts.draw();
    if ( pausef ) psLts.draw();
    if ( demoManager.demof ) {
      demoManager.display();
    }

    scene.glFunc.glPopMatrix();
  }

  private Random rnd = new Random();

  private void moveTimeLetters() {
    if ( tmlMv != 0 ) {
      tmlMv *= -0.9f;
      if ( tmlMv>-0.03f && tmlMv<0.03f ) tmlMv = 0;
      float x = tmlX + rnd.nextFloat()*tmlMv;
      float y = tmlY + rnd.nextFloat()*tmlMv;
      tmLts.set(x, y, TML_SIZE);
    }
  }

  private final float EFCLN_DST = 16;
  private float aelx, aely, aelz;
  private int elCngCnt;

  private void moveEffectLines() {
    if ( !bgefcf ) return;
    for ( int i=efcLns.length-1 ; i>=0 ; i-- ) {
      efcLns[i].move();
    }
    elx += (aelx-elx)*0.05f;
    ely += (aely-ely)*0.05f;
    elz += (aelz-elz)*0.05f;
    elCngCnt--;
    if ( elCngCnt <= 0 ) {
      int d = Math.abs(rnd.nextInt()%360);
      aelz = SCTable.sin[d]*EFCLN_DST;
      int d2 = Math.abs(rnd.nextInt()%360);
      aelx = SCTable.cos[d]*SCTable.sin[d2]*EFCLN_DST;
      aely = SCTable.cos[d]*SCTable.cos[d2]*EFCLN_DST;
      elCngCnt = Math.abs(rnd.nextInt()%300)+100;
    }
  }

  private final int[] CD_TIMES = {
    5*60000, 3*60000, 2*60000, 60000,
    30000, 20000, 10000, 5000, 4000, 3000, 2000, 1000, 0};

  private void sayCountDown() {
    if ( time <= CD_TIMES[seCnt] ) {
      playSe(6+seCnt);
      seCnt++;
    }
  }

  private void setTimeLetters() {
    tmLts.set();
    tmLts.add2(time/60000);
    tmLts.add('.');
    tmLts.add2((time/1000)%60);
    tmLts.add('.');
    tmLts.add2((time/10)%100);
  }

  private void gameOver() {
    time = 0;
    if ( bgmf ) {
      bgm.stop();
      bgm.setZeroTime();
    }
    if ( score > saveData.hiscore ) {
      saveData.hiscore = score;
    }
    demoManager.gameoverStart();
  }

  private boolean pskf = true;

  private boolean handlePause() {
    if ( sceneUI.psKey ) {
      if ( pskf ) {
        pausef = !pausef;
        pskf = false;
      }
    } else {
      pskf = true;
    }
    return pausef;
  }

  private void gameWakeUp() {
    if ( handlePause() ) return;

    if ( !ballManager.move() ) {
      time = 0; seCnt = CD_TIMES.length-1;
    }
    blockManager.move();
    racket.move();

    if ( gmStt ) {
      time -= DURATION_DEFAULT;
      if ( time <= 0 ) {
        gameOver();
      }
      setTimeLetters();
      sayCountDown();
    }

    decHitTime();
    htLts.move();
    taLts.move();
    mfLts.move();
    if ( mfLts.exists() ) {
      mfLts.y -= 0.02f;
    }
    if ( taLts.x < 10 ) {
      taLts.x += 0.5f;
      taLts.xScl += 0.1f;
      taLts.yScl += 0.1f;
    }
  }

  public final void wakeUp() {
    frmCnt++;
    if ( frmCnt > frameSkip ) {
      scene.display();
      frmCnt = 0;
    } else {
      waitFrame();
    }
    sceneUI.checkJoystick();

    moveTimeLetters();

    if ( demoManager.demof ) {
      demoManager.wakeUp();
    } else {
      gameWakeUp();
    }

    moveEffectLines();
    sePlayer.nextFrame();

    waitFrame();

    /*if ( sceneUI.ttKey ) {
      sceneUI.ttKey = false;
      stopBgm();
    }*/
  }
}

